<?php
declare(strict_types=1);

namespace Test\Test\Tests\Unit\Domain\Model;

use PHPUnit\Framework\MockObject\MockObject;
use TYPO3\TestingFramework\Core\AccessibleObjectInterface;
use TYPO3\TestingFramework\Core\Unit\UnitTestCase;

/**
 * Test case
 */
class ProductTest extends UnitTestCase
{
    /**
     * @var \Test\Test\Domain\Model\Product|MockObject|AccessibleObjectInterface
     */
    protected $subject;

    protected function setUp(): void
    {
        parent::setUp();

        $this->subject = $this->getAccessibleMock(
            \Test\Test\Domain\Model\Product::class,
            ['dummy']
        );
    }

    protected function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function getTitleReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getTitle()
        );
    }

    /**
     * @test
     */
    public function setTitleForStringSetsTitle(): void
    {
        $this->subject->setTitle('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('title'));
    }

    /**
     * @test
     */
    public function getColorReturnsInitialValueForColor(): void
    {
        $newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        self::assertEquals(
            $newObjectStorage,
            $this->subject->getColor()
        );
    }

    /**
     * @test
     */
    public function setColorForObjectStorageContainingColorSetsColor(): void
    {
        $color = new \Test\Test\Domain\Model\Color();
        $objectStorageHoldingExactlyOneColor = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $objectStorageHoldingExactlyOneColor->attach($color);
        $this->subject->setColor($objectStorageHoldingExactlyOneColor);

        self::assertEquals($objectStorageHoldingExactlyOneColor, $this->subject->_get('color'));
    }

    /**
     * @test
     */
    public function addColorToObjectStorageHoldingColor(): void
    {
        $color = new \Test\Test\Domain\Model\Color();
        $colorObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['attach'])
            ->disableOriginalConstructor()
            ->getMock();

        $colorObjectStorageMock->expects(self::once())->method('attach')->with(self::equalTo($color));
        $this->subject->_set('color', $colorObjectStorageMock);

        $this->subject->addColor($color);
    }

    /**
     * @test
     */
    public function removeColorFromObjectStorageHoldingColor(): void
    {
        $color = new \Test\Test\Domain\Model\Color();
        $colorObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['detach'])
            ->disableOriginalConstructor()
            ->getMock();

        $colorObjectStorageMock->expects(self::once())->method('detach')->with(self::equalTo($color));
        $this->subject->_set('color', $colorObjectStorageMock);

        $this->subject->removeColor($color);
    }
}
